################################################################ 


sub calculate_identity
{
  my ($filename, $redlimit, $gap, $Eblast) = @_;
  my $code1 = '';
  my $code2 = '';
  my $seq1 = '';
  my $seq2 = '';
  my $countN = 0;
  my $countGap = 0;
  my $countMatch = 0;
  my $countMismatch = 0; 

  unless (open(CLUSTAL, $filename))
    {
      print "Cannot open $filename\n";
      exit;
    }
  my @seqs = <CLUSTAL>;
  close(CLUSTAL);

# read the two sequences into two separate scalar 
  foreach my $ligne (@seqs)
  {
      if ( $ligne =~ /^[a-z0-9_]+\s{4,}/i )
      {
	  my $code = $&;
	  $code =~ s/\s//g;
	  # test if codes have been identified
	  unless ($code1 and $code2)
	  {
	      if ($code1)
	      {
		  $code2 = $code; 
	      }
	      else
	      {
	          $code1 = $code;
	      } 
	  }
	  if ($code eq $code1)
	  {
	      $ligne =~ /\s{4,}.+$/;
	      $seq1 .= $&;	      
	  }
	  else
	  {
	      $ligne =~ /\s{4,}.+$/;
	      $seq2 .= $&; 
	  }
      }
  }
  $seq1 =~ s/\s//g;
  $seq2 =~ s/\s//g;


# cut non-corresponding extremities
  if ($seq1 =~ /^-+/)
  {
    substr($seq1, 0, length $&, '');
    substr($seq2, 0, length $&, '');
  }

  if ($seq2 =~ /^-+/)
  {
    substr($seq1, 0, length $&, '');
    substr($seq2, 0, length $&, '');
  }

  if ($seq1 =~ /-+$/)
  {
    substr($seq1, length($seq1)-length $&, length $&, '');
    substr($seq2, length($seq2)-length $&, length $&, '');
  }

  if ($seq2 =~ /-+$/)
  {
    substr($seq1, length($seq1)-length $&, length $&, '');
    substr($seq2, length($seq2)-length $&, length $&, '');
  }

#  print "\n$code1\n$seq1\n";
#  print "$code2\n$seq2\n";

  for (my $position = 0; $position < length $seq1; ++$position)
  {
      my $base1 = substr($seq1, $position, 1); 
      my $base2 = substr($seq2, $position, 1); 
      if (($base1 eq 'N') or ($base2 eq 'N'))
      {
	  ++$countN;
      }
      elsif (($base1 eq '-') or ($base2 eq '-'))
      {
	  ++$countGap;
      }
      elsif ($base1 eq $base2)
      {
	  ++$countMatch;
      }
      else
      {
	  ++$countMismatch;
      }	     
  }

  my $booleen = '';
  my $identityGap = '';
  my $identityWOgap = '';

# If length difference is very important between two sequences Custal does not align well
  if (($countGap + $countMismatch + $countMatch) ==0) 
  {
      $identityGap = 0;
      $identityWOgap = 0;
  }
  else
  {
      $identityGap = ($countMatch) / ($countGap + $countMismatch + $countMatch);
      $identityWOgap =  ($countMatch) / ($countMismatch + $countMatch);
  }
#  print "\nIdentity $code1-$code2 with gaps: $identityGap\n";
#  print "\nIdentity $code1-$code2 without gaps: $identityWOgap\n";
      my $identity = 0;
      if ( $gap eq 'without')
      {
	  $identity = $identityWOgap;
      } 
      else
      {
	  $identity = $identityGap;
      }

      if ( $identity > $redlimit )
      {
	  $booleen = 1;
      }
      else
      {
	  $booleen = 0;
      }

  return ($booleen, $identityGap,$identityWOgap);

}



######################################################################

sub call_blast
{
    my ($filename, $Eblast, $path_blast, $outfolder) = @_;
    my $filename_out = $filename;
    $filename_out =~ s/\.[a-z0-9]+/\.txt/i; 
    $filename_out =~ s/.+\//$outfolder\//i; 

    my $formatdb = $path_blast.'formatdb -i '.$filename.' -p F -o T';
    my $blast = $path_blast.'blastall -p blastn -d '.$filename.' -i '.$filename.' -o '.$filename_out. ' -T F -e '.$Eblast;

    system $formatdb;
#    print "$formatdb\n";

    system $blast;
#    print "$blast\n"; 
    print "$filename_out: Results of the all-against-all BLASTn\n";
}

############################################################"


sub call_blast_vector
{
    my ($filename, $Evector, $path_blast) = @_;
    my $filename_out = $filename;
    $filename_out =~ s/\.[a-z0-9]+/\.vec/i; 
    my $formatdb = $path_blast.'formatdb -i Univec.fas -p F -o T';
    my $blast = $path_blast.'blastall -p blastn -d Univec.fas -i '.$filename.' -o '.$filename_out. ' -T F -e '.$Evector;
    system $formatdb;
#    print "$formatdb\n";

    system $blast;
#    print "$blast\n"; 
}


######################################################################

sub call_clustalw
{
    my ($script_file, $path_clustal, $outfolder) = @_;
    my $clustal = $path_clustal.'clustalw<'.$script_file.'>'.$outfolder.'/clustout';

    system $clustal;
   
}


#######################################################################


sub change_letters
 {
  use warnings;
  use strict;
   my ($seq_hach) = @_;
   my @code = keys %$seq_hach;
   my %NON_ATGC = ();

   foreach my $code (@code)
     {
       # delete spaces
       $$seq_hach{$code} =~ s/\s//g;
 
       # Check if other characters than letters are present

       # Change all letters to majuscules
       $$seq_hach{$code} = "\U$$seq_hach{$code}";

       $NON_ATGC{$code} = 'none';
       #puts non-permitted characters into a hachage
       while ($$seq_hach{$code} =~ /[^ATGCN]/g )
         {
	   $NON_ATGC{$code} .= "$&";
         }  
       if ($NON_ATGC{$code} =~ /[^a-z]/ )
       {
	   $NON_ATGC{$code} =~ s/none//;
       }

       # change all non (ATGC) into N
       $$seq_hach{$code} =~ s/[^ATCG]/N/g;
     }
   return %NON_ATGC;
 }

#########################################################

sub check_code_overlap
{
 my ($seq_hach, $outfolder) = @_;

 my $new_filename = '';
 my @codes_orig = sort (keys %$seq_hach);

# change evrything that is not letter number or underscore in codes into underscore

 my @codes = @codes_orig;
 foreach my $code (@codes)
 {
     while ( $code =~ /[^0-9a-z_]/ig )
     {
	 my $temp = $&;
	 print "'$temp' is replaced by '_' in $code\n";
	 $code =~ s/$temp/_/;
     }
 }

 
#check if none of the codes are substrings of other codes (e.g. A1 and A10)returns 1 if non-compatible codes, 0 if codes are fine 
 my $overlap = sub_string(\@codes);

# if incompatibility of codes, change all numbers in all codes into three digit format (e.g. change '11' into '011') or leave them intact if longer than three digits - else (remake seq_hach with only a-z0-9_ characters)
if ($overlap == 1)
{
    three_digit(\@codes);
#    print "@codes\n";
  
# Check again code compatibility (if incopatibility remane sequences with X letter Y digit code - else remake seq_hach with three digit codes)
  $overlap = sub_string(\@codes);
    if ($overlap == 1)
    {
	$new_filename = re_code_sequences(\%$seq_hach, $outfolder);
    }
    else
    {
	my %seq_hach_temp = ();
	for (my $i = 0; $i < scalar@codes; ++$i )
	{
	    $seq_hach_temp{$codes[$i]} = $$seq_hach{$codes_orig[$i]};
#	    print "$codes_orig[$i] - $codes[$i]\n";
	}
	%$seq_hach = %seq_hach_temp;
    }

}
else
{
    my %seq_hach_temp = ();
    for (my $i = 0; $i < scalar@codes; ++$i )
    {
	$seq_hach_temp{$codes[$i]} = $$seq_hach{$codes_orig[$i]};
#	print "$codes_orig[$i] - $codes[$i]\n";
    }
    %$seq_hach = %seq_hach_temp;
}

return ($new_filename);

}


#########################################################

sub check_reciproc_hits
{
 # keep only reciproc hits


  use warnings;
  use strict;

  my ($group_seq, $clustal) = @_;
  my @code = keys %$group_seq;


  foreach my $code (@code)
  {
      while ($$group_seq{$code} =~ /[a-z0-9_]+/ig)
      {
	  my $hit = $&;
	  if ( $$group_seq{$hit} )
	  {
	      if ( $$group_seq{$hit} !~ /$code/ )
	      {
	          $$group_seq{$code} =~ s/$hit,//;
		  $$clustal{$code} =~ /[a-z]*$hit,/i;
		  my $temp = $&;
		  $$clustal{$code} =~ s/$temp//;
 #                 print "$code-$hit is not reciproc\n";
	      }
	  }
	  else
	  {
	     $$group_seq{$code} =~ s/$hit,//;
	     $$clustal{$code} =~ /[a-z]*$hit,/i;
	     my $temp1 = $&;
	     $$clustal{$code} =~ s/$temp1//;
#	     print "$code-$hit is not reciproc\n";   
	  }	 
      }	  
  }

}

#########################################################


sub check_redundants
{
# Check all pairs of $code-$hits for redundancy, uses Clustal for alignement
# only overlaps are considered for counting the pairwise identities, and microsat 
# motifs are eliminated

my ($clustal, $filename, $outfolder, $redlimit, $gap, $path_clustal) =@_;
my @code = keys %$clustal;
my $tested_codes = '';
my $redundants = '';
my $idem_pairs = '';

# reads all seqences from the file (seqeunces with microsatellite motifs masqued) into a hachage
my %seq = fasta_into_hachage($filename);

# open outfile for pairwise identities

$filename =~ s/\..*/\.ide/i;
$filename =~ s/.+\//$outfolder\//i;
open (OUTFILE, ">$filename");
print OUTFILE "SEQ1;SEQ2;identity;\n";
  
 for (my $i = 0; $i < scalar(@code); ++$i)
  {
      my $code = $code[$i];
#      print "code:$code\n";
      # examine each hits of a code
      while ($$clustal{$code} =~ /[a-z0-9_]+/ig)
      {
#	  print "$$clustal{$code}\n";
	  my $hit_long = my $hit = $&; 
	  $hit =~ s/Minus//i;
          $hit =~ s/Plus//i;
#	  print "hit_long:$hit_long; hit:$hit\n";

          # eliminate $code of already tested pairs
	  if (($hit eq $code) or ($tested_codes =~ /$hit/) or ($redundants =~ /$hit/) or ($redundants =~ /$code/))
	  {
	      next;
	  } 

	  #Make fasta files with pairs of sequences, Takes into account the strand
	  my %temp_hach = ();
          $temp_hach{$code} = $seq{$code};
	  if ( $hit_long =~ /Minus/i )
	  {
	      my $temp = $seq{$hit};
#	      print "minus:$temp\n";
	      $temp_hach{$hit} = reverse_complement($temp);
#	      print "minus:$temp_hach{$hit}\n";
	  }
	  else
	  {
	      $temp_hach{$hit} = $seq{$hit};
#	      print "plus:$temp_hach{$hit}\n";
	  }
# cut out MS to avoid errors in Clustal is length is very different
	  while ( $temp_hach{$hit} =~ /N{8,}/ig )
	  {
	      my $Ns = $&;
	      $temp_hach{$hit} =~ s/$Ns//;
	  }
	  while ( $temp_hach{$code} =~ /N{8,}/ig )
	  {
	      my $Ns = $&;
	      $temp_hach{$code} =~ s/$Ns//;
	  }

#	  $filename =~ s/\..*//;
#	  $filename .= '.'.$code.$hit;
#	  print "$filename\n";
	  my $clustinp_file = $outfolder.'/clustinp';
	  make_fasta_file(\%temp_hach, $clustinp_file);
	  make_script_file($clustinp_file);
	  call_clustalw('script_file', $path_clustal, $outfolder);


	  
          # calculate identity returns 1 if identity is greater than 0.95
	  my $clustaln_file = $outfolder.'/clustinp.aln';
	  my ($booleen, $identityGap, $identityWOgap) = calculate_identity($clustaln_file, $redlimit, $gap, $Eblast);
          if ($booleen == 1)
	  {
	      if ( length $temp_hach{$hit} > length $temp_hach{$code} )
	      {
		  $redundants .= ' '.$code;
		  $idem_pairs .= ' '.$code.'-'.$hit;
	      }
	      else
	      {
		  $redundants .= ' '.$hit;
		  $idem_pairs .= ' '.$hit.'-'.$code;
	      }   
	  }  
	  print OUTFILE "$code;$hit;";	 
	  printf OUTFILE "%5.4f;\n",	  $identityGap;


      }
      $tested_codes .= ' '.$code;
   }

close OUTFILE;
print "$filename: Pair wise similarities between grouped sequences\n";

return ($redundants, $idem_pairs);
}


#########################################################


sub check_vector_MS_pos
{
#check if vector is situated before or after the microsat
# returns 0 if vector is before the Ms and 1 if after

    my ($HSPbeg, $HSPend, $MSbeg, $MSend, $code, $vector) = @_;
   
    if ( ($HSPend <= $MSbeg) and ($HSPbeg <= $MSbeg) )
      {
	return 0;
      }
    elsif ( ($HSPend >= $MSend) and ($HSPbeg >= $MSend) )
      {
	return 1;
      }
    else
      {
	   print "Overlap between Microsat in $code and Vector $vector\n";
	   return 2;
      }
      
}



##############################################################################



sub clean_file_1
{
 # delete empty lines and concatanate the diff lines of each sequences, put sequence into a hachage
  use warnings;
  use strict;
  my @seq = @_;
  my %seq_hach = ();
  my @code = ();
  my $code = '';
  my $temp = '';

#  reads table into a hachage; sequence names are the codes sequences are the values
  while (@seq)
  {
      my $seq = shift(@seq);
      if ($seq =~ /^>[a-z0-9_]+/i)
      {
	  $code = $&;
	  $code =~ s/>//;
   # change code into three letter three numbers code
	  $code =~ /[0-9]+/;

	  if (length $& == 1)
	  {
	      $temp = '00'.$&;
	      $code =~ s/$&/$temp/;
	  }
	  elsif (length $& == 2 )
	  {
	      $temp = '0'.$&;
              $code =~ s/$&/$temp/;
	  }


	  push (@code, $code);
      }
      else
      {
	  $seq_hach{$code} .= $seq;
      }
  }

  foreach my $code (@code)
  {
      $seq_hach{$code} =~ s/\s//sg;
  }

  return %seq_hach;
}


##############################################################################



sub clean_file_2
{
 # delete empty lines and concatanate the diff lines of each sequences, put sequence into a hachage
  use warnings;
  use strict;
  my @seq = @_;
  my %seq_hach = ();
  my @code = ();
  my $code = '';
  my $temp = '';

#  reads table into a hachage; sequence names are the codes sequences are the values
  while (@seq)
  {
      my $seq = shift(@seq);
      if ($seq =~ /^>.+?\s/)
      {
#	  print "**$seq****\n";
	  $code = $&;
#	  print "$code\n";
	  $code =~ s/>//;
	  $code =~ s/\s//;
#	  print "--$code----\n";
	  push (@code, $code);
      }
      else
      {
#	  print "$seq\n";
	  $seq_hach{$code} .= $seq;
      }
  }
#  print "@code\n";

  foreach my $code1 (@code)
  {
      $seq_hach{$code1} =~ s/\s//sg;
#      print "$code1    $seq_hach{$code1}\n";
  }

  return %seq_hach;
}



####################################################################
sub clean_seq_from_vector
{
    my ($vector_beg, $vector_end, $seqtemp) = @_;
    if ( $vector_end < 100000 )
    {
	$seqtemp = substr($seqtemp, 0, $vector_end);
    }
    if ( $vector_beg > 0 )
    {
	$seqtemp = substr($seqtemp, $vector_beg, length($seqtemp)-$vector_beg);
    }	
  return $seqtemp;
}

######################################################################

sub copie_hachage
{
# makes a copie of a hachage

my ($hachage, $hachage_copie) = @_;
my @code = keys %$hachage;

foreach my $code (@code)
{
    $$hachage_copie{$code} = $$hachage{$code};
}

}

#####################################################################



sub correct_for_non_reciproc
{
 my ($group_seq, $clas_seq, $clustal) = @_;
 my @code = keys %$group_seq;
 
 foreach my $code (@code)
 {
     my @data = split ( ',', $$group_seq{$code});

     if (scalar @data < 2 )
     {
	 $$clas_seq{$code} = 'unique';
	 $$group_seq{$code} = '';
	 $$clustal{$code} = '';
     }
 }


}

####################################################################


sub correct_redundants
{
  my ($clustal, $redundants, $idem_pairs) = @_;
  my @code = keys %$clustal;

  foreach my $code (@code)
  {
      $$clustal{$code} =~ s/Plus//g;
      $$clustal{$code} =~ s/Minus//g;
  }

  while ( $redundants =~ /[a-z0-9_]+/ig )
  {
      my $redundant_code = $&;
      $idem_pairs =~ /$redundant_code-[^\s]+/;
      my $idem = $&;
      $idem =~ s/$redundant_code-//;
     foreach my $code (@code)
      {
	  $$clustal{$code} =~ s/$redundant_code,//; 
      }      
      $$clustal{$redundant_code} = 'redundant_'.$idem;

 

# change seq into unique if all member of the group but one became redundant
      foreach my $code (@code)
      {
	     unless ($$clustal{$code} =~ /([a-z0-9_]+,){2,}/i)
	     {
		 unless ($$clustal{$code} =~ /redundant/)
		 {
		   $$clustal{$code} = 'unique';
	         }
	     }
      }
  }
}
######################################################################

sub delete_blast_file
{
  my ($folder) = @_;


#  my $boot = 'rm '.$folder.'bootstrap.*';
#  my $clustinp = 'rm '.$folder.'clust*';
#  my $won = 'rm '.$folder.'*.WON';
  my $blast = 'rm '.$folder.'/*.*.*';

#    system $boot;
#    system $clustinp;
#    system $won;
    system $blast;

}


##############################################################


sub delete_file
{
    my ($filename) = @_;

      my $del = 'rm '.$filename;
#    print "$del\n";
      system $del;
}


######################################################################

sub delete_temp_clust_file
{
  my ($folder) = @_;


#  my $boot = 'rm '.$folder.'bootstrap.*';
  my $clustinp = 'rm '.$folder.'/clust*';
#  my $won = 'rm '.$folder.'*.WON';
#  my $blast = 'rm '.$folder.'*.*.*';

#    system $boot;
    system $clustinp;
#    system $won;
#    system $blast;

}



################################################################
sub delete_Ns
  {
  use warnings;
  use strict;
    my ($seq_hach) = @_;
    my @code = keys %$seq_hach;
    my @seq = values %$seq_hach;
    my $n = 0;
    my %N_hach = ();

   foreach my $code (@code)
     {
       my $seq = $$seq_hach{$code}; 
       my $orig_length = length($seq);
       # delete ininterrupted Ns at the begining
       $seq =~ s/^N*//g;
      
       # if more than 2 Ns in the 10 bp at the begining delete positions till 
       # N (included), but keep ATGC
      
       do
         {
            $n = 0;
            my $windowbeg = substr ($seq, 0, 10);
            my @windowbeg = split('', $windowbeg);
  
            for (my $j = 0; $j < 10; ++$j)
               {
	          if ($windowbeg[$j] eq 'N' )
	          {
		    ++$n;
	          }
               }
            if ($n > 2)
	      {
	          $windowbeg =~ /[ACTG]*$/;
	          $windowbeg = $&; 
	      } 
            substr ($seq, 0, 10, $windowbeg);
	 } until ($n < 3);
       #count the number of bases cut at the beginning
       my $beg = $orig_length-length($seq);

       # if more than 2 Ns in the 10 bp at the end, delete positions beween the first
       # N (included) and the end

    # delete ininterrupted Ns at the end
       $seq =~ s/N*$//g;
       do
         {
            $n = 0;
            my $window = substr ($seq, length($seq)-10, 10);
            my @window = split('', $window);
  
            for (my $j = 0; $j < 10; ++$j)
               {
	          if ($window[$j] eq 'N' )
	          {
		    ++$n;
	          }
               }
            if ($n > 2)
	      {
	          $window =~ /^[ACTG]*/;
	          $window = $&; 
	      } 
            substr ($seq, length($seq)-10, 10, $window);
	 } until ($n < 3);
       #count the number of bases cut at the end
       my $end =  $orig_length-length($seq)- $beg;
       $N_hach{$code} .= "$beg $end";

        $$seq_hach{$code} = $seq;
    }
#  print "The number of bases deleted from the begining and from the end of the file are strored in N_hach\nseq_hach is over-written with the shortened sequences\n";

   return (%N_hach);
 }

####################################################################

sub delete_seq_WOmicrosat
{
    my ($seq_hach, $ms_hachage) = @_;
    my @code = keys %$seq_hach;
    my %seg_hach_WOmicrosat = ();


    foreach my $code (@code)
    {
	if ($$ms_hachage{$code} =~ /[0-9]/)
	{
	    $seq_hach_WOmicrosat{$code} = $$seq_hach{$code};
	}	
    }
    return %seq_hach_WOmicrosat;
}

#################################################################

sub extract_sequence_code
{
    my ($res_seq) = @_;
    $$res_seq =~ /^Query=.*/m;
    $code = $&;
    @code = split(/\s+/, $code);
    $code = $code[1];
    return $code;
}


#########################################################


sub extract_vector_res
{
# 
    my ($res_seq, $ms_hachage)= @_; 
    my $res_seq_vec = '';
    my $code = '';    
    my @code = ();
    my $vector = '';
    my $HSP = '';
    my $HSPbeg = '';
    my $HSPend = '';
    my $HSPend1 = '';
    my $vector_beg = 0;
    my $vector_end = 100000;
    my $vector_name_beg = '';
    my $vector_name_end = '';
    my $booleen = 0;

# extract sequence name
    $$res_seq =~ /^Query=.*/m;
    $code = $&;
    @code = split(/\s+/, $code);
    $code = $code[1];
# extract extrem positions of the Ms in the sequence
    my ($MSbeg, $MSend) = MS_extrem_pos(\%$ms_hachage, $code);
#    print "\n****$code*****\n";
#    print "\nMSpos:$MSbeg-$MSend\n";


# extract info sur une seq et une vector
 while ($$res_seq =~ />[^>]+/sg)
    {
	if ($booleen)
	{
	    next;
	}
	$res_seq_vec = $&;

#extract vector name	
	$res_seq_vec =~ /^>.+$/mi;
	$vector = $&;
#	print "\n/////$vector/////\n";

#extract HSP and HSP positions in sequence

	$res_seq_vec =~ s/Score/§core/g;
        while ($res_seq_vec =~ /§[^§]+/sg)
	{
	    if ($booleen)
	    {
		next;
	    }
	    $HSP = $&;
	    $HSP =~ /Query:\s[0-9]+/;
	    $HSPbeg = $&;
	    $HSPbeg =~ /[0-9]+/; 
	    $HSPbeg = $&;
	    while ($HSP =~ /^Query:.*$/gim)
                {
		   $HSPend = $&;
		}
      	    $HSPend =~ /[a-z]+\s+[0-9]+/;
	    $HSPend = $&;
	    $HSPend =~ /[0-9]+/;
	    $HSPend = $&;

	    my $i =check_vector_MS_pos($HSPbeg, $HSPend, $MSbeg, $MSend, $code, $vector);
#	    print "$HSPbeg-$HSPend-------$i\n";

#  keep only the most intern positions of the vector and the name of 
# the corresponding vector
	   
	    if ($i == 1)
	    {
		if ($HSPbeg <= $vector_end)
	        {
	             $vector_end = $HSPbeg;
	             $vector_name_end = $vector;
	        }
	    }
	    elsif ($i == 0)
	    {
		if ($HSPend >= $vector_beg)
		{
		    $vector_beg = $HSPend;	
		    $vector_name_beg = $vector;
		}    
	    }
	    else
	    {
		print "Vector:$vector, positions:$HSPbeg-$HSPend, MS:$MSbeg-$MSend, OVERLAP!!!\n";
		$vector_name_beg = 'overlap';
		$vector_name_end = 'overlap';
		$booleen = 1;
	    }
	}
    }
     
   
    return ($code, $vector_beg, $vector_name_beg, $vector_end, $vector_name_end);

 #   print "POSBEG: $vector_beg $vector_name_beg\n";
 #   print "POSEND: $vector_end $vector_name_end\n"; 

 }


####################################################################

sub fasta_into_hachage
{
 # delete empty lines and concatanate the diff lines of each sequences, put sequence into a hachage
  use warnings;
  use strict;
  my ($filename) = @_;
  my @seq = ();
  my %seq_hach = ();
  my @code = ();
  my $code = '';

  unless ( open(FASTA, $filename))
    {
      print "Cannot open $filename\n";
      exit;
    }
  @seq = <FASTA>;
  close(FASTA);

#  reads table into a hachage; sequence names are the codes sequences are the values
  while (@seq)
  {
      my $seq = shift(@seq);
      if ($seq =~ /^>[a-z0-9_]+/i)
      {
	  $code = $&;
	  $code =~ s/>//;
	  push (@code, $code);
      }
      else
      {
	  $seq_hach{$code} .= $seq;
      }
  }

  foreach my $code (@code)
  {
      $seq_hach{$code} =~ s/\s//sg;
  }

  return %seq_hach;
}



##################################################################
sub find_microsat_2
{
    my ($seq_hach, $min, $min_single) = @_;
    my @codes = keys %$seq_hach;
    my %ms_motif_pos = ();


foreach my $code (@codes)
{
    my $temp = '';
    my $seq = $$seq_hach{$code};
    my $bp = length $seq;
    $ms_motif_pos{$code} = 'motif: ';

    for (my $i= 0; $i < $bp; ++$i)
    {
#	print "$i ";
#Check for motif of single base pair
	my $motif = substr($seq, $i, 1);
	if ( $motif =~ /N/i )
	{
	    next;
	}
	my $j = $i+1;
	if ($j > $bp-1 )
	{
	    next;
	}
	while ( substr($seq, $j, 1) eq $motif )
	{
	    ++$j;
	}
	if ($j - $i > ($min_single-1))
	{
	    $temp = $i+1;
	    $ms_motif_pos{$code} .= $motif.' '.$temp.'-'.$j.', ';
	    $i = $j-1;
	    next;
	}
#Check for motif of two base pair
	$motif = substr($seq, $i, 2);
	if ( $motif =~ /N/i )
	{
	    next;
	}
	$j = $i+2;
	if ($j > $bp-2 )
	{
	    next;
	}	
	while ( substr($seq, $j, 2) eq $motif )
	{
	    $j +=2;
	}
	if ($j - $i > ($min*2-1))
	{
	    $temp = $i+1;
	    $ms_motif_pos{$code} .= $motif.' '.$temp.'-'.$j.', ';
	    $i = $j-1;
	    next;	    
	}
#Check for motif of three base pair

	$motif = substr($seq, $i, 3);
	if ( $motif =~ /N/i )
	{
	    next;
	}
	$j = $i+3;
	if ($j > $bp-3 )
	{
	    next;
	}	
	while ( substr($seq, $j, 3) eq $motif )
	{
	    $j +=3;
	}
	if ($j - $i > ($min*3-1))
	{
	    $temp = $i+1;
	    $ms_motif_pos{$code} .= $motif.' '.$temp.'-'.$j.', ';
	    $i = $j-1;
	    next;   
	}
#Check for motif of 4 base pair

	$motif = substr($seq, $i, 4);
	if ( $motif =~ /N/i )
	{
	    next;
	}
	$j = $i+4;
	if ($j > $bp-4 )
	{
	    next;
	}	
	while ( substr($seq, $j, 4) eq $motif )
	{
	    $j +=4;
	}
	if ($j - $i > ($min*4-1))
	{
	    $temp = $i+1;
	    $ms_motif_pos{$code} .= $motif.' '.$temp.'-'.$j.', ';
	    $i = $j-1;
	    next;	    
	}
#Check for motif of 5 base pair

	$motif = substr($seq, $i, 5);
	if ( $motif =~ /N/i )
	{
	    next;
	}
	$j = $i+5;
	if ($j > $bp-5 )
	{
	    next;
	}	
	while ( substr($seq, $j, 5) eq $motif )
	{
	    $j +=5;
	}
	if ($j - $i > ($min*5-1))
	{
	    $temp = $i+1;
	    $ms_motif_pos{$code} .= $motif.' '.$temp.'-'.$j.', ';
	    $i = $j-1;
	    next;	    
	}
#Check for motif of 6 base pair

	$motif = substr($seq, $i, 6);
	if ( $motif =~ /N/i )
	{
	    next;
	}
	$j = $i+6;
	if ($j > $bp-6 )
	{
	    next;
	}	
	while ( substr($seq, $j, 6) eq $motif )
	{
	    $j +=6;
	}
	if ($j - $i > ($min*6-1))
	{
	    $temp = $i+1;
	    $ms_motif_pos{$code} .= $motif.' '.$temp.'-'.$j.', ';
	    $i = $j-1;	 
	    next;
	}
    }

}    

return %ms_motif_pos;
}

##################################################################

sub first_group
{
    my ($filename, $clas_seq, $group_seq, $clustal, $outfolder) = @_;

# open file with the results of the Blast

    $filename =~ s/\.[a-z0-9]+/\.txt/i;
    $filename =~ s/.+\//$outfolder\//i;
    unless ( open(BLASTRES, $filename) )
    {
       print "Cannot open $filename!\n";
    }

#read the blast results for one sequence

    my $res_seq = '';
    while ($res_seq = one_seq_result($filename))
    {
       # avoide last scalar with only  a few characters 
       if ($res_seq !~ /Query/) 
       {
	   next;
       }

       my @subjects = ();
       my $subjects ='';
       my $strands = '';
       my $code = extract_sequence_code(\$res_seq);
 #  eliminate unBLASTable
       if ($res_seq =~ / \*{5}\sNo\shits\sfound\s\*{5}/ )
       {
	   $$clas_seq{$code} = 'unBLASTable';
       }
       else 
       { 
#extract subjects 
         while ($res_seq =~ />[^>]+/sg)
         {
	     my $res_seq_subj = $&;
	     $res_seq_subj =~ /^>.+$/mi;
	     my $subject = $&;
	     $subject =~ /[a-z0-9_]+/i;
	     $subject = $&;
	     push (@subjects, $subject);
	     $subjects .= "$subject,";
	     $res_seq_subj =~ /Strand.+$/mi;
	     my $strand = $&;
	     $strand =~ s/.+\/\s//;
	     $strand .= $subject;
	     $strands .= "$strand,";
	     
         }
         if ( scalar@subjects == 1 )
	 {
	    $$clas_seq{$code} = 'unique'; 
	 }
	 else
	 {
	    $$clas_seq{$code} = 'grouped'; 
	    $$group_seq{$code} = $subjects;
	    $$clustal{$code} = $strands;
	 }
       }   
    }
 close BLASTRES;

}


############################################################

sub length_sequences
{
    my ($hach) = @_;
    my @code = keys %$hach;
    my %orig_length  = ();

    foreach my $code (@code)
    {
	$orig_length{$code} = length($$hach{$code});
    }
    return %orig_length;
}

############################################################


sub make_fasta_file
  {
  use warnings;
  use strict;
   my ($seq_hach, $filename) = @_;
    my @code = sort (keys %$seq_hach);

    unless (open (FASTA, ">$filename"))
    {
	print "Cannot open $filename\n";
    }
    foreach my $code (@code)
    {
        my $seq = $$seq_hach{$code};
        print FASTA ">$code\n";
	for (my $pos = 0; $pos < length $seq; $pos += 50)
	{
	    print FASTA substr($seq, $pos, 50), "\n";
        }        

    }
  close (FASTA);

  }

############################################################


sub make_fasta_file_masqued
  {
  use warnings;
  use strict;
   my ($seq_hach, $ms_hachage, $filename) = @_;
    my @code = sort (keys %$seq_hach);

    open (FASTA, ">$filename");
    foreach my $code (@code)
    {
        my $seq = $$seq_hach{$code};
	my $replacement = $seq;
	$replacement =~ s/[a-z]/N/ig;
	while ( $$ms_hachage{$code} =~ /[0-9]+-[0-9]+,/g )
	{
	    my $temp = $&;
	    $temp =~ /[0-9]+-/;
            my $pos_beg = $&;
	    $pos_beg =~ s/-//;
	    $temp =~ /[0-9]+,/;
            my $pos_end = $&;
	    $pos_end =~ s/,//;
	    my $long = $pos_end - $pos_beg +1;
	    $replacement =~ /N{$long}/;
	    substr ($seq, $pos_beg-1, $long, $&); 
         }   
	print FASTA ">$code\n";
	for (my $pos = 0; $pos < length $seq; $pos += 50)
	{
	    print FASTA substr($seq, $pos, 50), "\n";
	}  
    }

  }


#############################################################

sub make_script_file
{
 my ($clustinp_file) = @_;

 open (SCR,">script_file" );
 print SCR "1\n$clustinp_file\n2\n1\n\n\nX\nX\nX\n";

 close SCR;

}

#############################################################

sub merge_groups_1
{

    my ($large_group) = @_;

    my @codes = sort (keys %$large_group);
    

    foreach my $code (@codes)
    {
#	print "\ncode:$code****\n";
	if ($$large_group{$code} =~ /redundant/)
	{
	    next;
	}
	my @hits = split ( ',', $$large_group{$code});
#	print "***@hits***\n";
	foreach my $hit (@hits)
	{
#	    print "code:$code,hit:$hit*\n";
	    for ( my $i = 0; $i < scalar@hits; ++$i )
	    {
#		print "$hit*";
		unless ( $$large_group{$hit} =~ /$hits[$i]/)
		{
		    $$large_group{$hit} .= $hits[$i].',';
		}
	    }
	}
    }

   # keep hits for only first member of the group, delete de value of the hachage for the others
    
    foreach my $code (@codes)
    {
	if ($$group_seq{$code =~ /redundant/})
	{
	    next;
	}
	my @hits = sort (split ( ',', $$large_group{$code}));
	for ( my $i = 1; $i < scalar@hits; ++$i )
	{
	    $$large_group{$hits[$i]} = '';
	}
    }
}

#####################################################################


sub MS_extrem_pos
{
# Extract from ms_hachage the begining of the first 
# MS and the end of the last
    my($ms_hachage, $code) = @_;
    my $MSbeg = 10000;
    my $MSend = 0;

   	while ( $$ms_hachage{$code} =~ /[0-9]+/g )
	{ 
	    my $temp = $&;
	    if ( $temp < $MSbeg )
	    {
		$MSbeg = $temp;
	    }
	    if ( $temp > $MSend )
	    {
		$MSend = $temp;
	    }
        }
    return ($MSbeg, $MSend);
}



###################################################################

sub numbering_large_group
{
 # Give number to each group and make a hachage 

  use warnings;
  use strict;

  my ($large_group_seq, $clas_seq) = @_;
  my @code = keys %$large_group_seq;
  

#  print "\n\n";

# eliminate redundant hits (hits that appear more than once) due to the 
# algotithme of the merge_group sub-program

  foreach my $code (@code)
  {
     my $buffer = '';
     while ( $$large_group_seq{$code} =~ /[a-z0-9_]+,/ig )
     {
	 $buffer .= $&;
	 $$large_group_seq{$code} =~ s/$&//ig;
     }
     $$large_group_seq{$code} = $buffer;
  }

# modify clas_seq (precise group number and correct for non-reciproc hits
 
  my $i = 1;
  foreach my $code (@code)
  {
      if ( $$large_group_seq{$code} =~ /redundant/ )
      {
	  next;
      }
      elsif ( $$large_group_seq{$code} =~ /^[a-z0-9_]+\s$/i )
      {
	  $$large_group_seq{$code} = 'unique';
      }
      elsif ( $$large_group_seq{$code} )
      {
	  while ( $$large_group_seq{$code} =~ /[a-z0-9_]+/ig)
	  {
	      my $temp = $&;
	      $$clas_seq{$temp} = "group$i";
	  }
	  ++$i;
      }
  }

}


##############################################################

sub print_hachage
 {
 # print out a hachage in a human readable format
  use warnings;
  use strict;   
   my ($hach) = @_;
   my @cles = keys %$hach;
   my @sorted_keys = sort @cles;

   foreach my $sorted_key (@sorted_keys)
     {
       print "$sorted_key     $$hach{$sorted_key}\n";
     }
  print "\n\n";
  }

###########################################################

sub print_hachage_into_file
 {
 # print out a hachage in a human readable format into a file
  use warnings;
  use strict;   
   my ($hach, $filename) = @_;
   my @code = keys %$hach;
   my @sorted_code = sort @code;

   open (HACH, ">$filename");
   foreach my $code (@sorted_code)
     {
        print HACH "$code   $$hach{$code}\n";
     }
  close (HACH);

  }

###########################################################

sub print_hachage_into_file_sep
 {
 # print out a hachage in a human readable format into a file
  use warnings;
  use strict;   
   my ($hach, $filename) = @_;
   my @code = keys %$hach;
   my @sorted_code = sort @code;

   open (HACH, ">$filename");
   foreach my $code (@sorted_code)
     {
        print HACH "$code;$$hach{$code};\n";
     }
  close (HACH);

  }
   
################################################################



sub print_clas_into_file
{
 my ($clas_seq, $group_seq, $clustal, $filename, $outfolder) = @_;

 $filename =~ s/\..*/\.clas/;
 $filename =~ s/.*\//$outfolder\//;
#  print "$filename\n";

unless (open (OUTFILE, ">$filename"))
{
    print "Cannot open $filename\n";
    exit;
}

# print_hachage(\%$clas_seq);

my @codes = sort (keys %$clas_seq);

print OUTFILE "CODE;GROUPS;with_REDUNDANTS;RAW_results;\n";

foreach my $code (@codes)
{
    if ($$clas_seq{$code} =~ /unique/i or $$clas_seq{$code} =~ /unBLASTable/i)
    {
	print OUTFILE "$code;$$clas_seq{$code};$$clas_seq{$code};$$clas_seq{$code};\n";
    }
    else
    {
	print OUTFILE "$code;$$clas_seq{$code};$$clustal{$code};$$group_seq{$code}\n";
    }
}

print "$filename: Categories for each sequences\n";

close OUTFILE;



}

###########################################################

sub recalculate_MS_pos_1
{
  my ($ms_hachage, $vector) = @_;
  my @code = keys %$vector;
  
  
  foreach my $code (@code)
  {
      $$vector{$code} =~ /[0-9]+/;
      my $vec_beg = $&;
      if ($vec_beg != 0 )
      {
	#  print "$code  $vec_beg\n";
	  my @temp_ms = split ( ' ', $$ms_hachage{$code});
	#  print "$$ms_hachage{$code}\n";
	  for (my $i = 0; $i < scalar @temp_ms; ++$i)
	  {
	      if ($temp_ms[$i] =~ /[0-9]+-/ )
	      {
		  my $ms_beg = $&;
		  $ms_beg =~ s/-//;
		  $ms_beg = $ms_beg - $vec_beg;
		  
		  $temp_ms[$i] =~ /[0-9]+,/;
                  my $ms_end = $&;
		  $ms_end =~ s/,//;
		  $ms_end = $ms_end - $vec_beg;
		  $temp_ms[$i] = $ms_beg.'-'.$ms_end.','
	      }
	  }
	  $$ms_hachage{$code} = join ( ' ', @temp_ms );
#	  print "$$ms_hachage{$code}\n";

      }
  }
}

##########################################################"

sub re_code_sequences
{
 my ($seq_hach, $outfolder) = @_;
 my @codes = sort (keys %$seq_hach);
 my %seq_hach_temp = ();
 my %code_hach = ();
 print "The codes of the sequences are incompatible with the program.\nSequences shall be renamed by a X letter Y digit code (e.g. PMN002).\nPlease, enter a short string (only letters) that will be the beginning of each of the codes:\n";
 my $code_beg = <STDIN>;
 chomp $code_beg;
 
 my $digit = length (scalar@codes);
 my $zeros = '0';
 while (length $zeros < $digit)
 {
     $zeros .= '0';
 }
 
 my $i = 1;
  foreach my $code (@codes)
 {
     my $number = $zeros;
     substr($number,($digit-(length$i)),length$i,$i);
     my $new_code = $code_beg.$number;
     $code_hach{$new_code} = $code;
     $seq_hach_temp{$new_code} = $$seq_hach{$code};
     ++$i;
 }

 %$seq_hach = %seq_hach_temp;
 my $filename = $outfolder.'/'.$code_beg.'.code'; 
 print_hachage_into_file_sep(\%code_hach, $filename);
 print "\n$filename: Correspondence between old and new codes\n";
 $filename =~ s/\..*/.fas/;
 make_fasta_file(\%$seq_hach, $filename);
 print "$filename: Fasta file with renamed but otherwise unchanged sequences\n";
 
 return ($filename);

}

##########################################################

sub reverse_complement
{
    my ($temp) = @_;
    my $revcomp = reverse $temp;
    $revcomp =~ tr/ATCGatcg/TAGCtagc/;
    return $revcomp;
}


##########################################################"

sub round_real
{
my ($decimal, $real) = @_;

$real =~ /[0-9]*\.[0-9]{,$decimal}/;
my $round = $&;
print "$round\n";
    return $round;

}


######################################################
sub sub_string
{
my ($codes) =@_;

 my $code_all = join ( ';', @$codes);
my $overlap = 0;
 foreach my $code (@$codes)
 {
     my $i = 0;
     while ($code_all =~ /$code/ig)
     {
	 ++$i;
     }
     if ($i > 1 )
     {
#	 print "$code\n";
	 $overlap = 1;
     }
 }

return $overlap;

}

#########################################################

sub three_digit
{
 my ($codes) = @_;
# print "@$codes\n";
    foreach my $code (@$codes)
    {
#	print "$code\n";
	my $code_temp = $code;
	  while ($code_temp =~ /[0-9]+/g)
	  {
#	      print "$&\n";
	      if (length $& == 2 )
	      {
		  $temp = '0'.$&;
		  
		  $code =~ s/$&/$temp/;
	      }
	      elsif (length $& == 1)
	      {
#		  print  "$&\n";
		  $temp = '00'.$&;
		  $code =~ s/$&/$temp/;
	      }
	  }
      }
# print "@$codes\n";
}

########################################################

sub write_info_separateur
{
my ($orig_length, $NON_ATGC, $N_hach, $vector, $clean_length, $ms_hachage, $filename) = @_;
my @code_total = keys %$N_hach;
my @sorted_code = sort @code_total;
my %vector_code = ();
my $i = 1;

my @short_code = keys %$vector;
my $short_code = join( ' ', @short_code ); 
$short_code .= ' ';
    
open (OUTFILE, ">$filename");

print OUTFILE "CODE;Orig_Length;NON_ATGC;Ns_BEG;Ns_END;Vector_pos_BEG;Vector_ref_BEG;Vector_pos_END;Vector_ref_END;New_Length;MICROSAT MOTIF and POS;\n";


foreach my $code (@sorted_code)
{
    # extract the number of bases deleted from the begining and from the end 
    #of the sequence due to Ns
    my @N = split( /\s+/, $$N_hach{$code} );
    my $Nend = pop @N;
    my $Nbeg = pop @N;
 
    my $vec_pos_beg = '';
    my $vec_pos_end = '';
    my $vec_beg = '';
    my $vec_end = '';
    my $ms = '';

    if ($short_code =~ /$code\s/)
    {
	$booleen = 1;
    
    # Extract the positions of the most far reaching vectors (0 and 10000 for no hits)
   
	$$vector{$code} =~ /BEG:\s[0-9]+/;
	$vec_pos_beg = $&;
	$vec_pos_beg =~ s/[^0-9]//g;

	$$vector{$code} =~ /SEND:\s[0-9]+/;
	$vec_pos_end = $&;
	$vec_pos_end =~ s/[^0-9]//g;

	# Extract vector names, gives code for vectors and puts the vector with the 
	# code into a hachage   
	$vec_beg = 'none';
	$vector_code{'none'} = 'none';
	if ($vec_pos_beg != 0)
	{
	    $$vector{$code} =~ />.+POSEND/;
	    $vec_beg = $&;
	    $vec_beg =~ s/\s+POSEND//;
	    unless( $vector_code{$vec_beg} )
	    {
		$vector_code{$vec_beg} = $i;
		++$i;
	    }
	}
	$vec_end = 'none';
	if ($vec_pos_end != 100000)
	{
	    $$vector{$code} =~ /POSEND.*/;
	    $vec_end = $&;
	    $vec_end =~ /POSEND.*>/;
	    my $temp = $&;
	    $vec_end =~ s/$temp/>/;
	    unless( $vector_code{$vec_end} )
	    {
		$vector_code{$vec_end} = $i;
		++$i;
	    }
	}


    #extract ms motifs and positions
    $ms = $$ms_hachage{$code};
    $ms =~ s/motif://;
#    $ms =~ s/\s//g;

    }

    else
    {
	$vec_pos_beg = 'NO MS';
	$vec_pos_end = 'NO MS';
        $vector_code{$vec_beg} = 'NO MS';
	$vector_code{$vec_end} = 'NO MS';
	$$clean_length{$code} = 'NO MS';
	$ms = 'NO MS';
    }   

   
print OUTFILE "$code;$$orig_length{$code};$$NON_ATGC{$code};$Nbeg;$Nend;$vec_pos_beg;$vector_code{$vec_beg};$vec_pos_end;$vector_code{$vec_end};$$clean_length{$code};$ms;\n"

}

my @cles = keys %vector_code;
my @valeurs = values %vector_code;
print OUTFILE "\n\n";
  while (@cles)
     {
       my $vector = shift(@cles);
       my $number = shift(@valeurs);
       if ($number !~ /[0-9]/)
       {next;}
       print OUTFILE "$number;$vector\n";
     }


close OUTFILE;


}

################################################################
1;
